<?php
/**
 * @author      Lefteris Kavadas
 * @copyright   Copyright (c) 2016 - 2018 Lefteris Kavadas / firecoders.com
 * @license     GNU General Public License version 3 or later
 */
defined('_JEXEC') or die;

class plgSystemRoute66 extends JPlugin
{
    private $canonical;

    public function onAfterInitialise()
    {
        $application = JFactory::getApplication();
        if ($application->get('sef')) {

            // Component params in use
            $componentParams = JComponentHelper::getParams('com_route66');
            $this->params->merge($componentParams);

            // Initialize our router
            include_once JPATH_SITE.'/plugins/system/route66/lib/router.php';
            $Route66 = new Route66SefRouter();
        }
    }

    public function onAfterDispatch()
    {
        $application = JFactory::getApplication();
        if ($application->isSite() && $application->get('sef')) {

            // Get document
            $document = JFactory::getDocument();

            // Get option
            $option = $application->input->getCmd('option');

            // Duplicate URLs handling
            if ($application->input->getMethod() == 'GET' && ($this->params->get('canonical', 1) || $this->params->get('redirect', 1)) && $document->getType() == 'html' && !in_array($option, $this->params->get('exclusions', array()))) {

                // Get router
                $router = $application->getRouter();

                // Get URI
                $uri = JUri::getInstance();

                // Merge all the available vars
                $vars = array_merge($router->getVars(), $uri->getQuery(true));
                $vars = array_filter($vars);

                // Canonical URI
                $url = $uri->getScheme().'://'.$uri->getHost();
                $port = $uri->getPort();
                if ($port) {
                    $url .= ':'.$port;
                }
                $url .= JRoute::_('index.php?'.http_build_query($vars), false);
                $canonical = JUri::getInstance($url);
                $query = $canonical->getQuery(true);
                $canonical = $canonical->toString(array('scheme', 'host', 'port', 'path'));

                // Current URI
                $current = $uri->toString(array('scheme', 'host', 'port', 'path'));

                // Canonical and current URI should match. If not redirect
                if ($this->params->get('redirect', 1) && $current != JUri::root() && urldecode($current) != urldecode($canonical)) {

                    // Redirect
                    $redirect = $canonical;
                    if (count($query)) {
                        $redirect .= '?'.http_build_query($query);
                    }
                    $application->redirect($redirect, 301);
                }

                // Add a canonical link
                if (!isset($query['start'])) {
                    $this->canonical = $canonical;
                }
            }
        }
    }

    public function onBeforeCompileHead()
    {
        $application = JFactory::getApplication();
        $document = JFactory::getDocument();
        if ($application->isSite() && $document->getType() == 'html') {
            if ($this->params->get('canonical', 1) && $this->canonical) {
                $document->addHeadLink(htmlspecialchars($this->canonical), 'canonical');
            }
            if ($this->params->get('facebookPageId')) {
                $document->addCustomTag('<meta property="fb:pages" content="'.$this->params->get('facebookPageId').'" />');
            }
        }
    }
}
